@@ -352,11 +352,17 @@ async def clean(ctx: Context, num_workers: int = 1, debug: bool = False):
     type=click.INT,
     help="Max number of workers to be spawned.",
 )
+@click.option(
+    "--seed",
+    "-s",
+    type=click.INT,
+    help="Seed for shuffling"
+)
 @click.pass_context
-async def prepare(ctx: Context, num_workers: int = 1):
+async def prepare(ctx: Context, seed: int, num_workers: int = 1):
     try:
         workflow = Workflow.load(ctx.obj["project"])
-        await workflow.run_stage("prepare", num_workers)
+        await workflow.run_stage("prepare", num_workers, seed)
         click.echo("---> Done")
     except Exception as e:
         click.echo(f"Error: {e}")
@@ -407,16 +413,21 @@ async def train(ctx: Context, gpu: int, continue_train: str):
     "--src", help="Source file for evaluation",
 )
 @click.option(
-    "--ref", help="Reference file"
+    "--ref", help="Reference file path"
+)
+@click.option(
+    "--out", help="Output file name"
 )
 @click.pass_context
-async def eval(ctx: Context, gpu: int, step: int, src: str, ref: str):
+async def eval(
+    ctx: Context, gpu: int, step: int, src: str, ref: str, out: str
+):
     try:
         workflow = Workflow.load(ctx.obj["project"])
         if src is not None:
             if ref is not None:
                 eval_stage = EvalStage(workflow)
-                await eval_stage.eval_files(gpu, step, src, ref)
+                await eval_stage.eval_files(gpu, step, src, ref, out)
             else:
                 print("Reference path missing")
         else:
