@@ -1,3 +1,6 @@
+import tempfile
+import time
+import shutil
 import subprocess
 import os
 import click
@@ -21,7 +24,8 @@ class PrepareStage(BaseStage):
     def __init__(self, workflow):
         super().__init__(workflow, self.NAME)
 
-    async def run(self, max_workers):
+    async def run(self, max_workers: int, seed: int):
+        start = time.ctime(int(time.time()))
         project = self.workflow.project
         project_dir = project.config.project_dir
 
@@ -29,22 +33,110 @@ class PrepareStage(BaseStage):
         workflow_dir = self.workflow.get_dir(project_dir)
         self.stage_dir = os.path.join(workflow_dir, PrepareStage.DIR)
 
-        if os.path.isdir(self.stage_dir):
-            raise Exception("Workflow already exists: " + str(project_dir))
+        if not os.path.isdir(self.stage_dir):
+            os.mkdir(self.stage_dir)
 
-        os.mkdir(self.stage_dir)
+        if max_workers is None:
+            max_workers = 1
 
-        await self.shuffle()
-        await self.tokenize(max_workers)
-        await self.truecase(max_workers)
-        await self.bpe(max_workers)
-        await self.prep_batch()
+        report = {}
 
-        return {}
+        workflow_config = self.workflow.config
+        to_the_end = False
+
+        if (
+            workflow_config.prepare["report"] is None
+            or "shuffle" not in workflow_config.prepare["report"]
+        ):
+            to_the_end = True
+            start_shuf = time.ctime(int(time.time()))
+            await self.shuffle(seed)
+            report["shuffle"] = {
+                "start": start_shuf,
+                "seed": seed,
+                "end": time.ctime(int(time.time()))
+            }
+            workflow_config.set_stage(
+                self.name,
+                {
+                    BaseStage.START: start,
+                    BaseStage.END: None,
+                    BaseStage.REPORT: report
+                }
+            )
+            workflow_config.save()
+
+        if to_the_end or "tokenize" not in workflow_config.prepare["report"]:
+            to_the_end = True
+            start_tokenize = time.ctime(int(time.time()))
+            await self.tokenize(max_workers)
+            report["tokenize"] = {
+                "start": start_tokenize,
+                "end": time.ctime(int(time.time()))
+            }
+            workflow_config.set_stage(
+                self.name,
+                {
+                    BaseStage.START: start,
+                    BaseStage.END: None,
+                    BaseStage.REPORT: report
+                }
+            )
+            workflow_config.save()
+
+        if to_the_end or "truecase" not in workflow_config.prepare["report"]:
+            to_the_end = True
+            start_truecase = time.ctime(int(time.time()))
+            await self.truecase(max_workers)
+            report["truecase"] = {
+                "start": start_truecase,
+                "end": time.ctime(int(time.time()))
+            }
+            workflow_config.set_stage(
+                self.name,
+                {
+                    BaseStage.START: start,
+                    BaseStage.END: None,
+                    BaseStage.REPORT: report
+                }
+            )
+            workflow_config.save()
+
+        if to_the_end or "bpe" not in workflow_config.prepare["report"]:
+            to_the_end = True
+            start_bpe = time.ctime(int(time.time()))
+            await self.bpe(max_workers)
+            report["bpe"] = {
+                "start": start_bpe,
+                "end": time.ctime(int(time.time()))
+            }
+            workflow_config.set_stage(
+                self.name,
+                {
+                    BaseStage.START: start,
+                    BaseStage.END: None,
+                    BaseStage.REPORT: report
+                }
+            )
+            workflow_config.save()
+
+        if (
+            to_the_end
+            or "prepare_batch" not in workflow_config.prepare["report"]
+        ):
+            to_the_end = True
+            start_prep_batch = time.ctime(int(time.time()))
+            await self.prep_batch()
+            report["prep_batch"] = {
+                "start": start_prep_batch,
+                "end": time.ctime(int(time.time()))
+            }
+
+        return report
 
     # Read the clean data and join both langs in a file to shuffle it without
     # losing the alignment
-    async def shuffle(self):
+    async def shuffle(self, seed: int):
         project = self.workflow.project
         project_dir = project.config.project_dir
 
@@ -53,10 +145,15 @@ class PrepareStage(BaseStage):
         cleaned_dir = os.path.join(workflow_dir, clean_stage.DIR)
 
         dir = os.path.join(self.stage_dir, "01_raw")
-        os.mkdir(dir)
+        try:
+            os.mkdir(dir)
+        except FileExistsError:
+            shutil.rmtree(dir)
+            os.mkdir(dir)
 
         all_lines_path = os.path.join(dir, "all_lines.txt")
         all_lines_shuf = os.path.join(dir, "all_lines_shuf.txt")
+        temp_path = tempfile.mkstemp()[1]
         all_lines = open(all_lines_path, "w+")
 
         # Reads the cleaned files and joins them in a file
@@ -74,11 +171,20 @@ class PrepareStage(BaseStage):
 
         # Shuffle the data
         print("Shufling and splitting data...")
-        command = (
+        command1 = (
             f'cat {all_lines_path} | sed -e "s/\r//g" | sed "s/\t//g" '
-            f"| sort | uniq | shuf > {all_lines_shuf}"
+            f"| sort | uniq > {temp_path}"
         )
-        subprocess.run(command, shell=True)
+        if seed is None:
+            seed = 3
+        command2 = (
+            f"seeded_shuffle.sh {temp_path} {str(seed)} > {all_lines_shuf}"
+        )
+
+        subprocess.run(command1, shell=True)
+        subprocess.run(command2, shell=True)
+
+        os.remove(temp_path)
 
         train_src = open(
             os.path.join(dir, "train_src.txt"), "w+", encoding="utf-8"
@@ -139,7 +245,11 @@ class PrepareStage(BaseStage):
     # Tokenize corpus
     async def tokenize(self, max_workers):
         dir = os.path.join(self.stage_dir, "02_tokenized")
-        os.mkdir(dir)
+        try:
+            os.mkdir(dir)
+        except FileExistsError:
+            shutil.rmtree(dir)
+            os.mkdir(dir)
 
         # Directory of the previous stage
         prev_dir = os.path.join(self.stage_dir, "01_raw")
@@ -195,7 +305,11 @@ class PrepareStage(BaseStage):
     # Apply the truecase to the data
     async def truecase(self, max_workers):
         dir = os.path.join(self.stage_dir, "03_truecased")
-        os.mkdir(dir)
+        try:
+            os.mkdir(dir)
+        except FileExistsError:
+            shutil.rmtree(dir)
+            os.mkdir(dir)
 
         prev_dir = os.path.join(self.stage_dir, "02_tokenized")
 
@@ -275,7 +389,11 @@ class PrepareStage(BaseStage):
             except Exception:
                 os.symlink(config.truecaser["pretrained"], dir)
         else:
-            os.mkdir(dir)
+            try:
+                os.mkdir(dir)
+            except FileExistsError:
+                shutil.rmtree(dir)
+                os.mkdir(dir)
 
             def make_path(file: str):
                 return os.path.join(prev_dir, file)
@@ -299,7 +417,11 @@ class PrepareStage(BaseStage):
     # Apply bpe to data
     async def bpe(self, max_workers):
         dir = os.path.join(self.stage_dir, "04_bpe")
-        os.mkdir(dir)
+        try:
+            os.mkdir(dir)
+        except FileExistsError:
+            shutil.rmtree(dir)
+            os.mkdir(dir)
 
         # Directory of the previous stage
         prev_dir = os.path.join(self.stage_dir, "03_truecased")
@@ -399,7 +521,11 @@ class PrepareStage(BaseStage):
             except Exception:
                 os.symlink(config.bpe["pretrained"], dir)
         else:
-            os.mkdir(dir)
+            try:
+                os.mkdir(dir)
+            except FileExistsError:
+                shutil.rmtree(dir)
+                os.mkdir(dir)
 
             iterations = self.workflow.project.config.bpe["num_iterations"]
 
@@ -426,7 +552,12 @@ class PrepareStage(BaseStage):
     # Split the corpus in shards
     async def prep_batch(self):
         dir = os.path.join(self.stage_dir, "05_batched")
-        os.mkdir(dir)
+        try:
+            os.mkdir(dir)
+        except FileExistsError:
+            shutil.rmtree(dir)
+            os.mkdir(dir)
+
         prev_dir = os.path.join(self.stage_dir, "04_bpe")
 
         args = [
